#define _WIN32_LEAN_AND_MEAN
#include <Windows.h>

#include <stdio.h>

#include "..\ZXpand_emu.h"

#include <vector>


typedef unsigned char BYTE;


// these don't need to be global, they just are.
//
void* instance;

ZXP_CREATEFN  zxp_create;
ZXP_DESTROYFN zxp_destroy;
ZXP_UPDATEFN  zxp_update;
ZXP_GETCFGFN  zxp_getcfg;
ZXP_IOWRFN    zxp_iowr;
ZXP_IORDFN    zxp_iord;


BYTE get_response(int port);


int PORT(int port)
{
   return port << 5;
}



static char asciiToZX[] = {
   0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
   0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,

   0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
   0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,

   0x00,0x0f,0x0b,0x0f,0x0d,0x0f,0x0f,0x0f,
   0x10,0x11,0x17,0x15,0x1a,0x16,0x1b,0x18,

   0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22,0x23,
   0x24,0x25,0x0e,0x19,0x13,0x14,0x12,0x0f,

   0x0f,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,
   0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x34,

   0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,
   0x3d,0x3e,0x3f,0x0f,0x18,0x0f,0x0f,0x0f,

   0x0f,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,
   0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x34,

   0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,
   0x3d,0x3e,0x3f,0x0f,0x0f,0x0f,0x0f,0x0f
};


static char ZXToAscii[] = " ??????????\"$:?()><=+-*/;,.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ\0";



std::vector<BYTE> ascii2zx(const char* source)
{
   size_t len = strlen(source);
   std::vector<BYTE> temp(len);

   for (size_t i = 0; i < len; ++i)
   {
      temp[i] = asciiToZX[source[i]];
   }

   return temp;
}


char zx2ascii(int source)
{
   return ZXToAscii[source & 0x3f];
}


int main(int argc, char **argv)
{
   // * Load the DLL
   HMODULE dll = LoadLibrary(TEXT("ZXpand_emu.dll"));
   if (!dll)
   {
      exit(0);
   }

   // * Create a set of pointers to the DLL functions
   zxp_create  = (ZXP_CREATEFN)  GetProcAddress(dll, "ZXpand_Create");
   zxp_destroy = (ZXP_DESTROYFN) GetProcAddress(dll, "ZXpand_Destroy");
   zxp_update  = (ZXP_UPDATEFN)  GetProcAddress(dll, "ZXpand_Update");
   zxp_getcfg  = (ZXP_GETCFGFN)  GetProcAddress(dll, "ZXpand_GetConfig");
   zxp_iowr    = (ZXP_IOWRFN)    GetProcAddress(dll, "ZXpand_IO_Write");
   zxp_iord    = (ZXP_IORDFN)    GetProcAddress(dll, "ZXpand_IO_Read");

   // * Create an instance of a ZXpand object
   instance = zxp_create("card.bin");
   if (instance == NULL)
   {
      printf("Couldn't load ZXpand_emu DLL\n");
      FreeLibrary(dll);
      exit(0);
   }



   //
   // The following is a test rig intended to show how functions are called. It bears
   // little relation to what you'd see in an emulator. This is more like the client code
   // you'd see in the operating system ROM.
   //
   // I'll assume that it's obvious that the iowr & iord calls should go in the relevant
   // opcode handler and that the update call will go in some function which is called
   // periodically.
   //



   // * Initialise the config object
   ZXpandConfig config;
   zxp_getcfg(instance, &config);
   printf("Initial configuration:\n  LOW = %s\n  DISABLED = %s\n", config.low ? "TRUE":"FALSE", config.disabled ? "TRUE":"FALSE");

   // change the config - disable the interface's overlay ROM
   zxp_iowr(instance, PORT(0x07), 0xb0);
   bool changed = zxp_getcfg(instance, &config);
   printf("Configuration changed = %s\n  LOW = %s\n  DISABLED = %s\n", changed ? "TRUE":"FALSE", config.low ? "TRUE":"FALSE", config.disabled ? "TRUE":"FALSE");

   //
   // get the directory listing off the card
   //

   printf("Do directory listing of <GM> folder:\n");

   // tell interface to prepare to receive data
   zxp_iowr(instance, PORT(0x00), 0xff);

   // make zx string version and send it to interface, zero terminated
   std::vector<BYTE> directoryname = ascii2zx("gm");
   for (size_t i = 0; i < directoryname.size(); ++i)
   {
      zxp_iowr(instance, PORT(0x02), directoryname[i]);
   }
   zxp_iowr(instance, PORT(0x02), 0);

   // now the directory name has been sent, we can command the interface to initialise the read
   // it's a slow command, we need to wait a while for it to complete
   //
   zxp_iowr(instance, PORT(0x03), 0);
   BYTE code = get_response(0x03);
   if (code != 0x40)
   {
      printf("Couldn't open directory\n");
      zxp_destroy(instance);
      FreeLibrary(dll);
      exit(0);
   }

   // now we can read items one by one from the directory

   // send 'read item' command - it will return 0x40 when OK, 0x3f when done, else error code
   //
   zxp_iowr(instance, PORT(0x03), 0xff);
   code = get_response(0x03);
   while (code == 0x40)
   {
      // there is an entry - read it from the buffer

      // tell interface to prepare to send data
      zxp_iowr(instance, PORT(0x00), 0);

      code = 0xff;
      while (code != 0)
      {
         code = zxp_iord(instance, PORT(0x01));
         printf("%c", zx2ascii(code));
      }
      printf("\n");

      zxp_iowr(instance, PORT(0x03), 0xff);
      code = get_response(0x03);
   }





   zxp_destroy(instance);

   FreeLibrary(dll);
}


// higher level ZXpand function - wait till the interface responds
// to a long function. bit 7 = busy.
//
BYTE get_response(int port)
{
   BYTE ret;
   do
   {
      // * repeatedly call update, passing elapsed mS since last call
      //   pretend there is a few mS delay between calls
      //
      zxp_update(instance, 3);
      ret = zxp_iord(instance, PORT(port));
   }
   while(ret > 127);

   return ret;
}
