// The following ifdef block is the standard way of creating macros which make exporting
// from a DLL simpler. All files within this DLL are compiled with the ZXPAND_DLL_EXPORTS
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see
// ZXPAND_DLL_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef ZXPAND_DLL_EXPORTS
#define ZXPAND_DLL_API __declspec(dllexport)
#else
#define ZXPAND_DLL_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif
   ZXPAND_DLL_API void* ZXpand_Create(const char* imagename);
   ZXPAND_DLL_API void  ZXpand_Destroy(void* instance);
   ZXPAND_DLL_API void  ZXpand_Update(void* instance, int millis);
   ZXPAND_DLL_API bool  ZXpand_GetConfig(void* instance, void* configDataPtr);
   ZXPAND_DLL_API void  ZXpand_IO_Write(void* instance, int addr, int data);
   ZXPAND_DLL_API int   ZXpand_IO_Read(void* instance, int addr);
#ifdef __cplusplus
};
#endif

typedef void*(*ZXP_CREATEFN)(const char*);
typedef void*(*ZXP_DESTROYFN)(void*);
typedef void(*ZXP_UPDATEFN)(void*,int);
typedef bool(*ZXP_GETCFGFN)(void*,void*);
typedef void(*ZXP_IOWRFN)(void*,int,int);
typedef int(*ZXP_IORDFN)(void*,int);

typedef struct
{
   bool low;      // =true means RAM at 8K, 16-40k, else ram at 16-48k.
   bool disabled; // =true means the overlay ROM is disabled.
}
ZXpandConfig;

//
// How to use the DLL
//
// Last update SirMorrisBig 20/05/2011
//

// * Load the DLL
//    HMODULE dll = LoadLibrary( TEXT("ZXpand_emu.dll") );
//
// * Create a set of pointers to the DLL functions
//    ZXP_CREATEFN  zxp_create  = (ZXP_CREATEFN)  GetProcAddress( dll, "ZXpand_Create" );
//    ZXP_DESTROYFN zxp_destroy = (ZXP_DESTROYFN) GetProcAddress( dll, "ZXpand_Destroy" );
//    ZXP_UPDATEFN  zxp_update  = (ZXP_UPDATEFN)  GetProcAddress( dll, "ZXpand_Update" );
//    ZXP_GETCFGFN  zxp_getcfg  = (ZXP_GETCFGFN)  GetProcAddress( dll, "ZXpand_GetConfig" );
//    ZXP_IOWRFN    zxp_iowr    = (ZXP_IOWRFN)    GetProcAddress( dll, "ZXpand_IO_Write" );
//    ZXP_IORDFN    zxp_iord    = (ZXP_IORDFN)    GetProcAddress( dll, "ZXpand_IO_Read" );
//
// * Create an instance of a ZXpand object
//    void* instance = zxp_create("card.bin");
//    if (instance == NULL) { [there has been an error] }
//
// * Initialise the interface state - ignore return on 1st call
//    ZXpandConfig config;
//    zxp_getcfg( instance, &config );.
//
// * Periodically call update on the object
//    zxp_update( instance, [number of milliseconds elapsed since the last update call] );
//
// * When an IO read occurs on port 7
//    data = zxp_ioread( instance, [address register bits 8..15] );
//
// * When an IO write occurs on port 7
//    zxp_iowrite( instance, [address register bits 8..15], data );
//
// * After an IO write you can check if the interface's config has changed
//    bool changed = zxp_getcfg( instance, &config ); // try to initialise the structure before subsequent calls
//    if (changed)
//    {
//       if (config.low) { [there is RAM at 8K, 16-40K] }
//       else { [there is RAM at 16-48K] }
//       if (config.disabled) { [interface overlay ROM is no longer present] }
//       else { [interface overlay ROM is present] }
//    }
//
// * When emulation stops or the machine is reset or before a new instance is needed (EG changing card image)
//    zxp_destroy( instance );
//    instance = NULL;
